jQuery(function ($) {
    var $cardElement = $('#card-element');

    if ($cardElement.length && window.Stripe) {
        var stripe = Stripe(quformStripe.publicKey);
        var elements = stripe.elements();
        var style = {
            base: {
                color: '#32325d',
                fontFamily: '"Helvetica Neue", Helvetica, sans-serif',
                fontSmoothing: 'antialiased',
                fontSize: '16px',
                '::placeholder': {
                    color: '#aab7c4'
                }
            },
            invalid: {
                color: '#fa755a',
                iconColor: '#fa755a'
            }
        };

        var card = elements.create('card', {style: style});
        card.mount($cardElement[0]);

        // Handle real-time validation errors from the card Element.
        card.addEventListener('change', function (event) {
            var displayError = document.getElementById('card-errors');
            if (event.error) {
                displayError.textContent = event.error.message;
            } else {
                displayError.textContent = '';
            }
        });

        var $form = $cardElement.closest('.quform-form');

        $form.off('submit').on('submit', function (e) {
            e.preventDefault();

            if ($cardElement.closest('.quform-element-page').hasClass('quform-current-page')) {
                stripe.createPaymentMethod({ type: 'card', card: card }).then(function (result) {
                    if (result.error) {
                        // Inform the user if there was an error.
                        var errorElement = document.getElementById('card-errors');
                        errorElement.textContent = result.error.message;
                    } else {
                        $form.find('#stripe_payment_method_id').val(result.paymentMethod.id);
                        $form.data('quform').submit();
                    }
                });
            } else {
                $form.data('quform').submit();
            }
        });

        $form.on('quform:errorEnd', function (e, form, response) {
            if(response.requires_action && response.payment_intent_client_secret && response.requires_action === 'handleCardAction') {
                stripe.handleCardAction(response.payment_intent_client_secret).then(function (result) {
                    if (result.error) {
                        // Inform the user if there was an error.
                        var errorElement = document.getElementById('card-errors');
                        errorElement.textContent = result.error.message;
                    } else {
                        $form.find('#stripe_payment_intent_id').val(result.paymentIntent.id);
                        $form.data('quform').submit();
                    }
                });
            }
        });

        $form.on('quform:successStart', function () {
            $form.find('#stripe_payment_method_id').val('');
            $form.find('#stripe_payment_intent_id').val('');
            card.clear();
        });
    }
});
