<?php

/*
 * Plugin Name: Quform Stripe
 * Description: Stripe integration for Quform.
 * Version: 2.0.1
 */

add_filter('quform_post_validate_1', function (array $result, Quform_Form $form) {
    require_once __DIR__ . '/vendor/autoload.php';

    if (empty($_POST['stripe_payment_method_id']) && empty($_POST['stripe_payment_intent_id'])) {
        $result = array(
            'type' => 'error',
            'error' => array(
                'enabled' => true,
                'title' => '',
                'content' => 'Please enter your card details to continue'
            )
        );

        // No JS response
        $form->setConfig(array(
            'errorEnabled' => true,
            'errorTitle' => '',
            'errorContent' => 'Please enable JavaScript and enter your card details to continue'
        ));

        return $result;
    }

    try {
        $client = new \Stripe\StripeClient('sk_test_key');

        if (!empty($_POST['stripe_payment_intent_id'])) {
            $paymentIntent = $client->paymentIntents->retrieve(sanitize_text_field(wp_unslash($_POST['stripe_payment_intent_id'])));
            $paymentIntent->confirm();
        } else {
            $paymentIntent = $client->paymentIntents->create([
                'amount' => 2000,
                'currency' => 'usd',
                'description' => 'Example product',
                'payment_method' => sanitize_text_field(wp_unslash($_POST['stripe_payment_method_id'])),
                'payment_method_types' => ['card'],
                'confirmation_method' => 'manual',
                'confirm' => true,
                'metadata' => [
                    'customer_email' => $form->getValueText('quform_1_3')
                ],
                'receipt_email' => $form->getValueText('quform_1_3')
            ]);
        }

        if ($paymentIntent->status == 'requires_action' && !empty($paymentIntent->next_action['type']) && $paymentIntent->next_action['type'] == 'use_stripe_sdk') {
            $result = array(
                'type' => 'error',
                'error' => array(
                    'enabled' => true,
                    'title' => '',
                    'content' => ''
                ),
                'requires_action' => 'handleCardAction',
                'payment_intent_client_secret' => $paymentIntent->client_secret
            );
        } elseif ($paymentIntent->status != 'succeeded') {
            throw new Exception('Payment failed');
        }
    } catch (Exception $e) {
        $result = array(
            'type' => 'error',
            'error' => array(
                'enabled' => true,
                'title' => '',
                'content' => $e->getMessage()
            )
        );
    }

    return $result;
}, 10, 2);

add_action('wp_enqueue_scripts', function () {
    wp_enqueue_script('stripe', 'https://js.stripe.com/basil/stripe.js', array(), false, true);
    wp_enqueue_script('quform-stripe', plugins_url('quform-stripe.js', __FILE__), array('stripe'), '2.0.1', true);
    wp_localize_script('quform-stripe', 'quformStripe', array(
        'publicKey' => 'pk_test_key'
    ));
}, 20);
